@@ -4,12 +4,28 @@ use std::{
     path::{Path, PathBuf},
 };
 
-use indicatif::{MultiProgress, ProgressBar, ProgressStyle};
+use indicatif::{MultiProgress, ProgressBar, ProgressFinish, ProgressStyle};
 use mktemp::Temp;
 
 const RETINA_SCALE: u32 = 2;
 
-pub const NUM_ICON_CONVERSION_STEPS: u64 = 13;
+pub enum ProgressBarType {
+    Input,
+    Conversion,
+    OutputWithIcns,
+    OutputWithoutIcns,
+}
+
+impl ProgressBarType {
+    pub fn num_steps(&self) -> u64 {
+        match self {
+            ProgressBarType::Input => 1,
+            ProgressBarType::Conversion => 13,
+            ProgressBarType::OutputWithIcns => 7,
+            ProgressBarType::OutputWithoutIcns => 1,
+        }
+    }
+}
 
 use crate::{
     command::{
@@ -56,29 +72,33 @@ impl WorkingDir {
 
     pub fn icon_conversion(
         &self,
-        resolution_prefix: &str,
+        progress_bar_type: ProgressBarType,
+        stage_description: &str,
         multi_progress_bar: Option<MultiProgress>,
     ) -> IconConversion {
         let progress_bar = match multi_progress_bar {
             Some(multi_progress_bar) => {
-                let progress_bar =
-                    multi_progress_bar.insert(0, ProgressBar::new(NUM_ICON_CONVERSION_STEPS));
-                progress_bar.set_prefix(format!("[{}]", resolution_prefix));
+                let progress_bar = ProgressBar::new(progress_bar_type.num_steps());
+                let progress_bar = match progress_bar_type {
+                    ProgressBarType::Conversion => multi_progress_bar.insert(1, progress_bar),
+                    _ => multi_progress_bar.insert_from_back(0, progress_bar),
+                };
+                let progress_bar = progress_bar.with_finish(ProgressFinish::AndLeave);
                 // TODO share the progress bar style?
                 let progress_bar_style = ProgressStyle::with_template(
-                    "{prefix:12} | {bar:12.cyan/blue} | {pos:>2}/{len:2} | {msg}",
+                    "{bar:12.cyan/blue} | {pos:>2}/{len:2} | {wide_msg}",
                 )
                 .expect("Could not construct progress bar.")
                 .progress_chars("=> ");
                 progress_bar.set_style(progress_bar_style);
-                progress_bar.set_message("Preparing");
+                progress_bar.tick();
                 Some(progress_bar)
             }
             None => None,
         };
         IconConversion {
             working_dir: self.working_dir.as_path().to_owned(),
-            resolution_prefix: resolution_prefix.into(),
+            resolution_prefix: stage_description.into(),
             progress_bar,
         }
     }
@@ -242,9 +262,16 @@ pub struct IconInputs {
 }
 
 impl IconConversion {
-    fn step(&self, step_desciption: &str) {
+    pub fn step_unincremented(&self, step_description: &str) {
         if let Some(progress_bar) = &self.progress_bar {
-            progress_bar.set_message(step_desciption.to_owned());
+            let wide_msg = format!("{:10} | {}", self.resolution_prefix, step_description);
+            progress_bar.set_message(wide_msg);
+        }
+    }
+
+    pub fn step(&self, step_desciption: &str) {
+        if let Some(progress_bar) = &self.progress_bar {
+            self.step_unincremented(step_desciption);
             progress_bar.inc(1);
         }
     }
@@ -260,6 +287,7 @@ impl IconConversion {
         options: &options::Options,
         centering_dimensions: &Dimensions,
     ) -> Result<PathBuf, FolderifyError> {
+        self.step_unincremented("Preparing icon mask");
         let mut args = CommandArgs::new();
         args.background_transparent();
         args.density(density(&options.mask_path, centering_dimensions)?);
@@ -273,6 +301,7 @@ impl IconConversion {
         let output_path = self.output_path("0.0_FULL_MASK.png");
         args.push_path(&output_path);
         run_convert(&args, None)?;
+        self.step("");
         Ok(output_path)
     }
 
@@ -427,6 +456,8 @@ impl IconConversion {
         args.push_path(output_path);
         run_convert(&args, Some(template_icon))?;
 
+        self.step("");
+
         Ok(())
     }
 
@@ -445,7 +476,7 @@ impl IconConversion {
         let size = inputs.resolution.size();
         let offset_y = inputs.resolution.offset_y();
 
-        self.step("Sizing mask");
+        self.step_unincremented("Sizing mask");
         let sized_mask_path = self
             .sized_mask(
                 full_mask_path,
@@ -503,6 +534,7 @@ impl IconConversion {
         iconset_dir: &Path,
         icns_path: &Path,
     ) -> Result<(), FolderifyError> {
+        self.step("Creating .icns file");
         if options.verbose {
             println!(
                 "[{}] Creating the .icns file...",
@@ -543,12 +575,14 @@ impl IconConversion {
         };
 
         // sips: add an icns resource fork to the icns file
+        self.step("Adding resource fork to .icns file using sips");
         let mut args = CommandArgs::new();
         args.push("-i");
         args.push_path(icns_path);
         run_command(SIPS_COMMAND, &args, None)?;
 
         // DeRez: export the icns resource from the icns file
+        self.step("Exporting .icns resource using DeRez");
         let mut args = CommandArgs::new();
         args.push("-only");
         args.push("icns");
@@ -562,6 +596,7 @@ impl IconConversion {
         }
 
         // Rez: add exported icns resource to the resource fork of target/Icon^M
+        self.step("Add .icns resource target using Rez");
         let mut args = CommandArgs::new();
         args.push("-append");
         args.push_path(&derezzed_path);
@@ -570,6 +605,7 @@ impl IconConversion {
         run_command(REZ_COMMAND, &args, None)?;
 
         // SetFile: set custom icon attribute
+        self.step("Setting custom icon attribute");
         let mut args = CommandArgs::new();
         args.push("-a");
         args.push("-C");
@@ -577,14 +613,19 @@ impl IconConversion {
         run_command(SETFILE_COMMAND, &args, None)?;
 
         if target_is_dir {
+            self.step("Setting invisible file attribute");
             // SetFile: set invisible file attribute
             let mut args = CommandArgs::new();
             args.push("-a");
             args.push("-V");
             args.push_path(&target_resource_path);
             run_command(SETFILE_COMMAND, &args, None)?;
+        } else {
+            self.step("Skipping invisible file attribute for file target");
         }
 
+        self.step("");
+
         Ok(())
     }
 }
