@@ -55,6 +55,9 @@ cargo run -- --color-scheme light ./examples/src/apple.png
 echo -e "\nTest that \`--color-scheme dark\` is accepted."
 cargo run -- --color-scheme dark ./examples/src/apple.png
 
+echo -e "\nTest that \`--no-progress\` is accepted."
+cargo run -- --no-progress ./examples/src/apple.png
+
 echo -e "\nTest that \`--output-icns\` is accepted."
 cargo run -- --output-icns ./examples/src/folder_outline_custom_path_1.icns ./examples/src/folder_outline.png
 check_file ./examples/src/folder_outline_custom_path_1.icns
@@ -74,9 +77,38 @@ check_folder ./examples/src/folder_outline_custom_path_4.iconset
 check_no_file_nor_folder ./examples/src/folder_outline.icns
 check_no_file_nor_folder ./examples/src/folder_outline.iconset
 
-# for version in "10.5" "10.6" "10.7" "10.8" "10.9" "10.10" "10.11" "10.12" "10.13" "10.14" "10.15" "11.0"
-for version in "10.15" "13.3"
+for version in "10.5" "10.8" "10.15"
 do
-  echo -e "\nTest that --macOS ${version} is accepted."
-  cargo run -- --macOS ${version} ./examples/src/apple.png
+  echo -e "\nTest that --macOS ${version} is rejected."
+  # Wrap command to avoid triggering `pipefail`.
+  if (cargo run -- --macOS ${version} ./examples/src/apple.png)
+  then
+      failure "Not rejected."
+  else
+      success "Rejected (expected)."
+  fi
+done
+
+# Accepted with a warning.
+for version in "10.16" "99.0"
+do
+  echo -e "\nTest that --macOS ${version} is accepted with a warning."
+  # Wrap command to avoid triggering `pipefail`.
+  if (cargo run -- --macOS ${version} ./examples/src/apple.png 2>&1 | grep "Warning: Unknown macOS version specified\.")
+  then
+    success "Accepted with warning."
+  else
+    failure "Command failed or warning missing."
+  fi
+done
+
+for version in "11.0" "12.1" "13.3"
+do
+  echo -e "\nTest that --macOS ${version} is accepted without warning"
+  if (cargo run -- --macOS ${version} ./examples/src/apple.png 2>&1 | grep "Warning: Unknown macOS version specified\.")
+  then
+    failure "Command failed or unexpected warning."
+  else
+    success "Accepted without warning."
+  fi
 done
